/******************************************************************************
          Copyright (C) 1990, Photometrics, Ltd. All Rights Reserved
******************************************************************************/

/******************************************************************************
                        CCD Characterization Structure
******************************************************************************/
typedef struct {
  unsigned cols;              /* Number of Columns (Serial Shifts) in CCD */
  unsigned rows;              /* Number of Rows (Parallel Shifts) in CCD */
  unsigned prescan;           /* Number of serial shifts before data */
  unsigned postscan;          /* Number of serial shifts after data */
  unsigned parallel_delay;    /* Parallel Delay=0.6 + parallel_delay * 0.1 us */
  unsigned parallel_states;   /* Number of states for a parallel clock */
  unsigned frame_transfer:1;  /* TRUE if frame transfer version is available */
  unsigned mpp_expose:1;      /* TRUE if MPP exposure is available */
  unsigned mpp_clock:1;       /* TRUE if MPP clocking is available */
  unsigned high_hwell:1;      /* TRUE if HWELL must be high for exposure */
  char     id[16];            /* Name of the CCD */
} ccd_descript;

#define CCD_COUNT   12        /* The number of CCDs in the table */
extern ccd_descript ccd[CCD_COUNT]; /* The table of standard CCDs */

/******************************************************************************
                      AT200 Primitive Control Structures
******************************************************************************/
/******************************************************************************
                 AT200 Register Offsets from the base address
******************************************************************************/
#define FIFO_DATA               0   /* Read/Write = FIFO data */
#define STATUS                  1   /* Read       = Status Register */
#define RESERVED1               2   /* Not Used */
#define CONTROL                 3   /* Read/Write = Control Register */
#define RESERVED2               4   /* Not Used */
#define TTL_IO                  5   /* Read/Write = TTL I/O Port */
#define SER_CONTROL             6   /* Write      = Serial/Parallel Converter */
#define SAFE_WRITE              7   /* Write      = No Effect */
/******** The DSP registers below do not respond if the DSP is reset ********/
#define DSP_INTERRUPT_CONTROL   8   /* Read/Write = DSP ICR */
#define DSP_COMMAND_VECTOR      9   /* Read/Write = DSP Command Vector */
#define DSP_INTERRUPT_STATUS   10   /* Read       = DSP Interrupt Status */
#define DSP_INTERRUPT_VECTOR   11   /* Read/Write = DSP Interrupt Vector */
#define DSP_NULL               12   /* Read       = 0, Write = No Effect */
#define DSP_XFER_HIGH          13   /* Read/Write = DSP High Transfer Byte */
#define DSP_XFER_MIDDLE        14   /* Read/Write = DSP Middle Transfer Byte */
#define DSP_XFER_LOW           15   /* Read/Write = DSP Low Transfer Byte */
/******************************************************************************
                    AT200 Status Register Bit Definitions
******************************************************************************/
#define FIFO_OVERFLOW   128   /* STATUS & FIFO_OVERFLOW = 0 -> Fatal Error */
#define FIFO_EMPTY       64   /* Clear FIFO_OVERFLOW latch by reseting FIFO */
#define FIFO_HALF_FULL   32
#define EXTERNAL_TRIGGER 16   /* Latched. 0=Trigger. DSP_INT_DISABLE=0 Clears. */
/******************************************************************************
                    AT200 Control Register Bit Definitions
******************************************************************************/
#define FIFO_RESET          1 /* Set to 0 to reset FIFO, 1 for FIFO operation */
#define DSP_RESET           4 /* Set to 0 to reset DSP, 1 for DSP operation */
#define DSP_INT_DISABLE     8 /* Set to 1 for External Trigger operation */
#define NON_VOLATILE_SAVE  16 /* Set to 1 to write to non-volatile RAM */
#define FIFO_INPUT_SELECT  32 /* Select FIFO input: 0=camera, 1=FIFO_DATA Reg.*/
/******************************************************************************
               AT200 Non-Volatile RAM Register Bit Definitions
******************************************************************************/
#define ADDRESS_MASK        0x1F0 /* Mask for Storable Address Bits */
#define ADDRESS_SHIFT(c) ((c)>>4) /* Shift right 4 Bits */
#define AUTO_ADDRESS_OFF       32 /* Bit to turn off auto address seek mode */
/* Non-Volatile RAM will be written with
  (ADDRESS_SHIFT(address & ADDRESS_MASK) | AUTO_ADDRESS_OFF)
  to fix its position in the I/O map.  Writing 0 enables auto address mode. */
/******************************************************************************
                DSP Interrupt Control Register Bit Definitions
******************************************************************************/
#define RECEIVE_REQUEST_ENABLE  1   /* Enables ~HREQ pin for host interrupts */
#define TRANSMIT_REQUEST_ENABLE 2   /* Enables ~HREQ pin for host interrupts */
#define ICR_RESERVED            4   /* Set to Zero. */
#define HOST_FLAG_0             8   /* Host flag Zero (DSP can only read) */
#define HOST_FLAG_1            16   /* Host flag One (DSP can only read) */
#define HOST_MODE_0            32   /* Host Mode Bit Zero */
#define HOST_MODE_1            64   /* Host Mode Bit One */
#define HOST_INTERFACE_INIT   128   /* Host Interface DMA Initialize */
/******************************************************************************
               DSP Host Command Vector Register Bit Definitions
******************************************************************************/
#define COMMAND_MASK    0x1F    /* 32 Vectors Supported */
#define COMMAND_PENDING  128    /* Write=Vector Request, Read=Vector Acknowledged */
/******************************************************************************
          DSP Interrupt Status Register Bit Definitions (Read Only)
******************************************************************************/
#define DSP_RECEIVE_DATA_FULL   1   /* DSP data available to Host */
#define DSP_TRANSMIT_DATA_EMPTY 2   /* Last Host data is in DSP memory */
#define DSP_TRANSMIT_READY      4   /* Last Host data is in DSP CPU */
#define HOST_FLAG_2             8   /* Host flag Two */
#define HOST_FLAG_3            16   /* Host flag Three */
#define ISR_RESERVED           32   /* Read as Zero. */
#define DSP_DMA_STATUS         64   /* 1=DSP is in a DMA mode */
#define DSP_HOST_REQUEST      128   /* 1=DSP Requested Host Interrupt */
/******************************************************************************
                             External Definitions
******************************************************************************/
extern int at200_base;  /* I/O Map Base Address of the AT200 */
void collect_image(unsigned far * far ptable[]); /* Fast Collect into DOS RAM */
int  continuous_clear(void); /* Starts continuous clearing of the CCD */
void disable_trigger(void); /* Disables the External Trigger to the DSP */
void dsp_arg(long argument); /* Sends an argument to the Host Interface */
int  dsp_command(int cmd); /* Sends a Command Vector to the DSP */
void dsp_op(int cmd,unsigned argument); /* Sends an interpreter op & argument */
int  emm_present(void); /* Returns TRUE if EMM detected, FALSE otherwise */
int  emm_version(void); /* Returns EMM version as 0xYZ. Ver=Y.Z */
int  ems_alloc (long mem_size); /* Allocates EMS pages from the EMM */
int  ems_assign_name(int ems_handle,char far *name); /* Names a handle EMM V4.0 */
void ems_collect_image(int ems_handle); /* Fast Collect into EMS RAM */
int  ems_find_named_handle(char far *name); /* Returns a named handle EMM V4.0 */
void ems_free (int ems_handle); /* Deallocates EMS pages previously allocated */
int  ems_get_name(int ems_handle,char far *name); /* Returns the name EMM V4.0 */
char far *ems_get_page_frame(void); /* Returns ptr to EMS page frame or NULL. */
int  ems_handle_size(int ems_handle); /* Returns pages in the handle */
int  ems_handle_resize(int ems_handle,int pages); /* Changes page count EMM V4.0 */
int  ems_map_page(int physical_page,int ems_handle,int page); /* Ret. -1 on error */
int  ems_pages_available(void); /* Returns the number of unallocated EMS pages */
int  ems_size_check(int ems_handle,long size); /* Reallocs if needed EMM V4.0 */
void enable_trigger(void); /* Enables the External Trigger to the DSP */
int  load_dsp_program(unsigned char far *program,int bytes); /* Returns NULL on error */
void reset_fifo(void); /* Clears the FIFO and readies it for operation */
